
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "matrix.h"
#include "indeps.h"


// Number of variables. Defined once at init, then everything
// is done with this fixed n.
static int n;

// Other static variables defined at init time.
static long long twoPowerNminus2;
static long long nIndep;
static long long twoPowerN;

// Static matrix to hold the transformation from pair to index
static matrix pairIndexToVarPair;

// Static matrix to hold size of conditioning sets for the
// independences (to avoid computing these many times)
static matrix indepcondsetsize;


// Initialization. Do this first!
void InitIndeps( int nvars )
{
  // Init number-of-variable-related stuff
  n = nvars;
  twoPowerNminus2 = ((long long) pow((double) 2,n-2));
  nIndep = (n*(n-1)/2) * twoPowerNminus2;
  twoPowerN = ((long long) pow((double) 2,n));
  
  // Init the matrix to hold transformation from pair to index
  pairIndexToVarPair = CreateMatrix( n*(n-1)/2, 2, 0);
  int k = 0;
  for (int i=0; i<n; i++)
  {
    for (int j=i+1; j<n; j++)
    {
      el(pairIndexToVarPair, k, 0) = i;
      el(pairIndexToVarPair, k, 1) = j;
      k++;
    }
  }
  
  // Compute the size of the conditioning sets
  indepcondsetsize = CreateMatrix(nIndep, 1, -1);
  matrix ivec = CreateMatrix(1, n, 0);
  for (long long ii=0; ii<nIndep; ii++)
  {
    int condsetsize = 0;
    IndepIndexToIndepVec( ii, ivec );
    for (int i=0; i<n; i++)
    {
      if (el(ivec,0,i)==1) condsetsize++;
    }
    el(indepcondsetsize,ii,0) = condsetsize;
  }
  
}

int IndepCondSetSize( long long ii )
{
  return el(indepcondsetsize,ii,0);
}

long long nTotIndep()
{
  return nIndep;
}

void IndepIndexToIndepVec( long long ii, matrix vec )
{
  // First compute the pair.
  int k = (ii / twoPowerNminus2);
  int i = el(pairIndexToVarPair, k, 0);
  int j = el(pairIndexToVarPair, k, 1);
    
  // Compute the subset index
  long long si = (ii % twoPowerNminus2);
  
  // Next fill out the vector
  long long fact = 2;
  for (int v=0; v<n; v++)
  {
    // Default value is zero
    el(vec,0,v)=0;
    
    if ((v == i) || (v == j))
    {
      // Variable is part of the pair
      el(vec,0,v)=2;      
    }
    else
    {
      // Fill out the conditioning set part
      long long rem = si % fact;
      if (rem)
      {
        el(vec,0,v)=1;
        si -= rem;
      }
      fact *= 2;
    }
  }
  
  // Return 
  return;
}

long long IndepVecToIndepIndex( matrix ivec )
{
  // Step through the vector, get i and j and si
  int i=-1;
  int j=-1;
  long long si=0;
  long long fact=1;
  for (int k=0; k<n; k++)
  {
    if (el(ivec,0,k) == 2)
    {
      if (i<0) i=k; else j=k;
    }
    else
    {
      si += (el(ivec,0,k) * fact);
      fact *= 2;
    }
  }
  long long k = i*n - i - i*(i-1)/2 + j - i - 1;
  long long ii = k*twoPowerNminus2 + si;
  
  return ii;
}


long long nTotSub()
{
  return twoPowerN;
}

void SubIndexToSubVec( long long si, matrix vec )
{
  // Fill out the vector
  long long fact = 2;
  for (int v=0; v<n; v++)
  {
    // Default value is zero
    el(vec,0,v)=0;
    
    long long rem = si % fact;
    if (rem)
    {
      el(vec,0,v)=1;
      si -= rem;
    }
    fact *= 2;
  }
  
  return;
}

long long SubVecToSubIndex( matrix vec )
{
  // Step through the vector, compute si
  long long si=0;
  long long fact=1;
  for (int k=0; k<n; k++)
  {
    si += (el(vec,0,k) * fact);
    fact *= 2;
  }
  
  return si;
}

int SubSize( matrix vec )
{
  int s = 0;
  int n = vec.cols;
  for (int i=0; i<n; i++)
  {
    if (el(vec, 0, i) == 1) s++;
  }
  return s;
}

