
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include "matrix.h"

static int rformat = 0;

// function for initializing a matrix
matrix CreateMatrix( int r, int c, int val )
{
  matrix M;
  int i;
  
  // allocate the memory and set rows and cols variables
  M.contents = malloc( r * c * sizeof(int) );
  M.rows = r;
  M.cols = c;
  
  // initialize it to the value specified
  for (i=0; i<(r*c); i++) M.contents[i] = val;
  
  // return the matrix
  return M;
}

// function for setting all elements of an existing matrix
void SetMatrix( matrix M, int val )
{
  int r = M.rows;
  int c = M.cols;
  
  // set all elements
  for (int i=0; i<(r*c); i++) M.contents[i] = val;
  
  // return the matrix
  return;
}

// function for copying a matrix
matrix CopyMatrix( matrix M )
{
  int r = M.rows;
  int c = M.cols;
  
  // allocate the memory and set rows and cols variables
  matrix X; 
  X.contents = malloc( r * c * sizeof(int) );
  X.rows = r;
  X.cols = c;

  // copy the data
  for (int i=0; i<(r*c); i++) X.contents[i] = M.contents[i];
  
  // return the matrix
  return X;
}

// function for removing a matrix
void RemoveMatrix( matrix M )
{
  free( M.contents );
  return;
}

int IdenticalMatrix( matrix M1, matrix M2 )
{
  if (M1.rows != M2.rows) return 0;
  if (M1.cols != M2.cols) return 0;
  
  for (int i=0; i<(M1.rows*M1.cols); i++)
  {
    if (M1.contents[i] != M2.contents[i]) return 0;
  }
  
  return 1;
}

// function to set rformat
void SetRformat( int setrformat )
{
  rformat = setrformat;
}


// function to print a matrix
void PrintMatrix( char *msg, matrix M )
{
  int i,j;

  printf("%s", msg );
  printf("\n");
    
  if (rformat)
  {
    printf("t(matrix(c(");
  }
  
  for (i=0; i<M.rows; i++)
  {
    for (j=0; j<M.cols; j++)
    {
      printf("%i",el(M,i,j));
      if (rformat) 
      {
        if ((i<(M.rows-1)) || (j<(M.cols-1)))
        {
          printf(",");
        }
      }
      else printf(" ");
    }
    printf("\n");
  }
  if (!rformat) printf("\n");
  if (rformat)
  {
    printf("), %i, %i)) \n",M.cols, M.rows);
  }
  
  return;
}

// prints out the transpose of a matrix for visual inspection
// (NOTE: when using R format, the output of this function
// does not in any way differ from the output of PrintMatrix)
void PrintTransposeMatrix( char *msg, matrix M )
{
  int i,j;

  if (rformat)
  {
    // if we are printing out the matrix for reading into R,
    // it does not visually matter how it is printed
    PrintMatrix( msg, M );
    return;
  }
  
  printf("%s", msg );
  printf("\n");
  
  for (i=0; i<M.cols; i++)
  {
    for (j=0; j<M.rows; j++)
    {
      printf("%i ",el(M,j,i));
    }
    printf("\n");
  }
  printf("\n");
  
  return; 
}


