/****************************************************************************************[Helpers.h]
Copyright (c) 2012-2013 by Patrik Hoyer

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
associated documentation files (the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT
OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
**************************************************************************************************/

#ifndef Minisat_Helpers_h
#define Minisat_Helpers_h

// Writes to stdout
void writeResponse(int cmdNum, char *response);

// Gets the next command from the command file
int getNextCmd(char *cmd);

// Sets the reordering type
void setReorderType(int rtype);

// Sets up the initial permutation array
void initPerm();

// Checks if a given external variable has an internal representation already
// If it does, it simply returns the internal variable number. If it does not, 
// it returns 0.
int getInternVar( long long externVar );

// Creates a new internal variable to hold the specified external one
int createInternVar( long long externVar );

// Handles the external->internal mapping using a tree
int trie_lookup_or_add( long long x, int addflag );

// A node in the tree structure that stores the external to internal mapping
typedef struct tnode_struct {
  int y;
  struct tnode_struct *ch[10];
} tnode;

// Max line length of commands (sufficient to hold all the supported commands, so 
// is not restrictive in any sense)
#define MAXCMDLEN  100

#endif
